/**
 * {}
 * @author yutent<yutent.io@gmail.com>
 * @date 2023/03/16 18:05:43
 */
import { css, html, Component, bind, unbind, nextTick } from 'wkit'
import '../icon/icon.js'

class Passwd extends Component {
  static props = {
    value: 'str!',
    autofocus: false,
    readonly: false,
    disabled: false,
    placeholder: 'str!',
    lazy: 'num!0' // 并发拦截时间, 单位毫秒
  }

  static styles = [
    css`
      :host {
        overflow: hidden;
        display: inline-flex;
        min-width: 188px;
        color: var(--color-dark-1);
        border-radius: 3px;
        cursor: text;
        transition: box-shadow 0.15s linear;
        -webkit-user-select: none;
        user-select: none;
      }

      .container {
        flex: 1;
        display: flex;
        justify-content: center;
        align-items: center;
        height: 32px;
        font-size: 14px;
        border: 1px solid var(--wc-passwd-border-color, var(--color-grey-2));
        border-radius: inherit;
        background: var(--wc-passwd-background, #fff);
        color: inherit;
        cursor: inherit;

        input {
          flex: 1;
          min-width: 36px;
          width: 0;
          height: 100%;
          padding: 0 4px 0 8px;
          border: 0;
          border-radius: inherit;
          color: inherit;
          font-family: inherit;
          font-size: inherit;
          background: none;
          outline: none;
          box-shadow: none;
          cursor: inherit;

          &::placeholder {
            color: var(--color-grey-2);
          }
        }
      }
      :host([round]) .container input {
        padding: 0 6px 0 20px;
      }
      :host(:focus-within) {
        box-shadow: 0 0 0 2px var(--color-plain-a);
      }
    `,
    css`
      .container {
        .icon {
          --wc-icon-size: 16px;
          margin: 0 8px 0 4px;
          color: var(--color-grey-3);
          cursor: pointer;
        }
      }
    `,
    // 尺寸
    css`
      :host([size='small']) {
        min-width: 128px;
        .container {
          height: 24px;
        }
      }
    `,
    css`
      :host([disabled]),
      :host([readonly]) {
        cursor: default;

        .icon {
          cursor: inherit;
        }

        .container {
          background: var(--color-readonly-background);
          opacity: 0.6;
        }
      }
      :host([disabled]) {
        cursor: not-allowed;

        .container {
          background: var(--color-disabled-background);
        }
      }
    `
  ]

  #type = 'password'
  #stamp = 0

  mounted() {
    if (this.autofocus) {
      this.$refs.input.focus()
    }
  }

  #toggleView(ev) {
    if (this.readOnly || this.disabled) {
      return
    }
    this.#type = this.#type === 'password' ? 'text' : 'password'
    this.$requestUpdate()
  }

  #fetch(ev) {
    let now = Date.now()
    let { lazy } = this
    this.value = ev.target.value

    // 并发拦截
    if (lazy && now - this.#stamp < lazy) {
      return
    }

    this.#stamp = now
    this.$emit(ev.type)
  }

  render() {
    return html`
      <main class="container">
        <slot class="prepend" name="prepend"></slot>
        <input
          spellcheck="false"
          ref="input"
          placeholder=${this.placeholder}
          @input.stop=${this.#fetch}
          @change=${this.#fetch}
          :readOnly=${this.readOnly}
          :disabled=${this.disabled}
          :type=${this.#type}
          :value=${this.value}
        />
        <wc-icon
          class="icon"
          @click=${this.#toggleView}
          :name=${this.#type === 'password' ? 'eye' : 'eye-off'}
        ></wc-icon>
      </main>
    `
  }
}

Passwd.reg('passwd')
